@@ -3,46 +3,47 @@
 
 import os
 from pprint import pprint
-
 # import awswrangler as wr
 from subprocess import check_output
 
 import fire
 
-from .utils.aws import (
-    cancel_step,
-    describe_cluster,
-    describe_step,
-    get_step_state,
-    list_steps,
-    print_emr_log_files_lines,
-    ssm_cluster,
-    download_emr_logs
+from pyemr.utils.build import build, submit_spark_step
+from pyemr.utils.config import init_pyemr
+from pyemr.utils.docker import launch_docker_bash, launch_docker_shell
+from pyemr.utils.emr import (
+    cancel_step, describe_cluster, describe_step, get_clusters_list_df,
+    get_step_state, list_steps, ssm_cluster,
 )
-from .utils.build import (
-    build,
-    pack_poetry_project_in_docker,
-    submit_spark_step,
-    upload_amazonlinux_build_s3,
-    upload_file_s3,
+from pyemr.utils.export import export_airflow_template
+from pyemr.utils.linting import format_code, lint_wd, spell_check
+from pyemr.utils.logs import (
+    download_all_emr_logs, print_emr_log_files_lines, summarize_logs,
 )
-from .utils.config import add_pyemr_param, init_pyemr
-from .utils.docker import (
-    launch_docker_bash,
-    launch_docker_notebook,
-    launch_docker_python,
-    launch_docker_shell,
-    launch_pyspark,
-    local_spark_submit,
+from pyemr.utils.mocking import mock_part_s3_folder, mock_s3_folder
+from pyemr.utils.notebook import (
+    launch_mock_notebook_docker, run_notebook_in_poetry, run_notebook_on_sys,
+)
+from pyemr.utils.python import (
+    launch_mock_python_docker, launch_mock_python_sys, launch_mock_python_venv,
+)
+from pyemr.utils.sys import os_cmd
+from pyemr.utils.testing import (
+    test_script_with_s3_mock_docker, test_script_with_s3_mock_sys,
+    test_script_with_s3_mock_venv,
 )
-from .utils.emr import get_clusters_list_df
 
 
 class Cli:
     """Command line interface for the package."""
 
     def init(
-        self, project_name="", target_cluster="", s3_stage_dir="", stage="", region=""
+        self,
+        project_name="",
+        target_cluster="",
+        s3_stage_dir="",
+        stage="",
+        region="",
     ):
         """Create a pyproject.toml containing pyemr config.
 
@@ -82,18 +83,6 @@ class Cli:
         """
         ssm_cluster(cluster_name)
 
-    def ssmbash(self, cluster_name, cmd):
-        """run a command on a cluster master node
-
-        Args:
-          cluster_name:
-          cmd:
-
-        Returns:
-
-        """
-        # ssm_bash(cluster_name, cmd)
-
     def build(self, cluster_name=""):
         """Zips the python package and dependencies and then uploads them s3 staging directory.
 
@@ -117,7 +106,12 @@ class Cli:
         self.build(cluster_name)
 
     def submit(
-        self, script, submit_mode="client", cluster_name="", wait=True, **kwargs
+        self,
+        script,
+        submit_mode="client",
+        cluster_name="",
+        wait=True,
+        **kwargs,
     ):
         """
 
@@ -133,7 +127,7 @@ class Cli:
 
         """
         submit_spark_step(script, submit_mode, cluster_name, wait, **kwargs)
-
+    
     def config(self, toml_path="pyproject.toml"):
         """
 
@@ -146,55 +140,43 @@ class Cli:
         input_dir = os.getcwd()
         check_output(["open", f"{input_dir}/{toml_path}"])
 
-    def docker_build(self):
-        """ """
-        pack_poetry_project_in_docker()
-
-    def upload_build(self):
-        """ """
-        print(upload_amazonlinux_build_s3())
-
-    def bsubmit(self, location="s3"):
-        """
-
-        Args:
-          location: (Default value = "s3")
-
-        Returns:
-
-        """
-        self.buildsubmit(location)
-
-    def notebook():
-        """Starts a spark linux notebook in current directory"""
-
-    def pyspark(self):
-        """Starts a pyspark session in doker with the current director booted."""
-
-    def export(self, type):
+    def export(
+        self,
+        local_script_path,
+        submit_mode="client",
+        action_on_failure="CONTINUE",
+        *args,
+        **kwargs,
+    ):
         """
 
         Args:
           type:
+          local_script_path:
+          submit_mode: (Default value = 'client')
+          action_on_failure: (Default value = 'CONTINUE')
+          *args:
+          **kwargs:
 
         Returns:
 
         """
-        assert type in ["aws", "bash", "python", "airflow", "awswrangler"]
-    
+        # assert type in ["aws", "bash", "python", "airflow", "awswrangler"]
+        export_airflow_template(
+            local_script_path,
+            submit_mode="client",
+            action_on_failure="CONTINUE",
+            *args,
+            **kwargs,
+        )
 
-    def s3(self, cmd):
-        """
-
-        Args:
-          cmd:
-
-        Returns:
-
-        """
-
-    def list_steps(
-        self, cluster_name="", n=10, step_name="{env_name}:*", states="*", all=False
+    def steps(
+        self,
+        cluster_name="",
+        n=10,
+        step_name="{env_name}:*",
+        states="*",
+        all=False,
     ):
         """
 
@@ -212,8 +194,12 @@ class Cli:
             step_name = states = "*"
         pprint(list_steps(cluster_name, n, step_name, states))
 
-    def cancel_step(
-        self, step_id="", cluster_name="", step_name_pattern="{env_name}:*", state="*"
+    def cancel(
+        self,
+        step_id="latest",
+        cluster_name="",
+        step_name_pattern="{env_name}:*",
+        state="*",
     ):
         """
 
@@ -229,7 +215,11 @@ class Cli:
         pprint(cancel_step(cluster_name, step_id, step_name_pattern, state))
 
     def describe_step(
-        self, step_id="", cluster_name="", name="{env_name}:*", state="*"
+        self,
+        step_id="latest",
+        cluster_name="",
+        name="{env_name}:*",
+        state="*",
     ):
         """
 
@@ -243,11 +233,10 @@ class Cli:
 
         """
         pprint(describe_step(cluster_name, step_id, name, state))
-    
-    def logs(
+
+    def stderr(
         self,
-        log_type,
-        step_id="",
+        step_id="latest",
         n=30,
         cluster_name="",
         name="*{env_name}*",
@@ -257,7 +246,6 @@ class Cli:
         """
 
         Args:
-          log_type:
           step_id: (Default value = "")
           n: (Default value = 30)
           cluster_name: (Default value = "")
@@ -269,37 +257,19 @@ class Cli:
 
         """
         print_emr_log_files_lines(
-            log_type, n, step_id, cluster_name, name, state, out_dir
+            "stderr",
+            n,
+            step_id,
+            cluster_name,
+            name,
+            state,
+            out_dir,
+            "FAIL",
         )
-        # shutil.rmtree(out_dir)
-
-    def stderr(
-        self,
-        step_id="",
-        n=30,
-        cluster_name="",
-        name="*{env_name}*",
-        state="*",
-        out_dir="logs",
-    ):
-        """
-
-        Args:
-          step_id: (Default value = "")
-          n: (Default value = 30)
-          cluster_name: (Default value = "")
-          name: (Default value = "*{env_name}*")
-          state: (Default value = "*")
-          out_dir: (Default value = "./logs")
-
-        Returns:
-
-        """
-        self.logs("stderr", step_id, n, cluster_name, name, state, out_dir)
 
     def stdout(
         self,
-        step_id="",
+        step_id="latest",
         n=30,
         cluster_name="",
         name="*{env_name}*",
@@ -319,9 +289,18 @@ class Cli:
         Returns:
 
         """
-        self.logs("stdout", step_id, n, cluster_name, name, state, out_dir)
+        print_emr_log_files_lines(
+            "stdout",
+            n,
+            step_id,
+            cluster_name,
+            name,
+            state,
+            out_dir,
+            "OKCYAN",
+        )
 
-    def state(self, step_id="", cluster_name="", name="*{env_name}*", state="*"):
+    def state(self, step_id="latest", cluster_name="", name="*{env_name}*", state="*"):
         """
 
         Args:
@@ -346,133 +325,205 @@ class Cli:
         """
         pprint(describe_cluster(cluster_name))
 
-    def list_clusters(self, states=["RUNNING", "WAITING"], n=10):
-        """
+    def clusters(self, states="RUNNING|WAITING", n=10):
+        """List the clusters in the given states.
 
         Args:
-          states: (Default value = ['RUNNING')
-          'WAITING']:
+          states: (Default value = "RUNNING|WAITING" )
           n: (Default value = 10)
-          "WAITING"]:
 
         Returns:
 
         """
         print(get_clusters_list_df(states, n))
 
-    def ls(
+    def notebook(
         self,
-        entity_type,
-        step_name_pattern="*{env_name}*",
-        states=["RUNNING", "WAITING"],
-        n=10,
-        *args,
-        **kwards,
+        env: str = "docker",
     ):
-        """
+        """Runs docker notebooks with s3 patch.
 
         Args:
-          entity_type:
-          step_name_pattern: (Default value = "*{env_name}*")
-          states: (Default value = ['RUNNING')
-          'WAITING']:
-          n: (Default value = 10)
-          *args:
-          **kwards:
-          "WAITING"]:
+          env: str:  (Default value = 'docker')
+          env: str:  (Default value = "docker")
 
         Returns:
 
         """
+        assert env in ["sys", "local", "os", "poetry", "venv", "docker", "linux"]
 
-        if entity_type in "clusters":
-            self.list_clusters(*args, **kwards)
-        if entity_type in "steps":
-            self.list_steps(*args, **kwards)
+        if env in ["sys", "local", "os"]:
+            run_notebook_on_sys()
 
-    def s3_cp(self, local_path, out_dir="."):
-        """
+        if env in ["venv", "poetry"]:
+            run_notebook_in_poetry()
+
+        if env in ["docker", "linux"]:
+            launch_mock_notebook_docker()
+
+    def venv_python(self):
+        """Runs interactive python session with s3 patch inside the virtual env."""
+        launch_mock_python_venv()
+
+    def python(self, env: str = "docker"):
+        """Runs interactive docker python session with s3 patch.
 
         Args:
-          local_path:
-          out_dir: (Default value = '.')
+          env: str:  (Default value = "docker")
 
         Returns:
 
         """
-        print(upload_file_s3(local_path, out_dir))
+        assert env in ["sys", "local", "os", "poetry", "venv", "docker", "linux"]
 
-    def set(self, key, value):
-        """
+        if env in ["sys", "local", "os"]:
+            launch_mock_python_sys()
 
-        Args:
-          key:
-          value:
+        if env in ["poetry", "venv"]:
+            launch_mock_python_venv()
 
-        Returns:
-
-        """
-        add_pyemr_param(key, value)
-
-    def notebook(self):
-        """ """
-        launch_docker_notebook()
-
-    def python(self, *args, **kwards):
-        """
-
-        Args:
-          *args:
-          **kwards:
-
-        Returns:
-
-        """
-        launch_docker_python(*args, **kwards)
+        if env in ["docker", "linux"]:
+            launch_mock_python_docker()
 
     def sh(self):
-        """ """
+        """Stats a docker shell session."""
         launch_docker_shell()
 
     def bash(self):
-        """ """
+        """Stats a docker bash session."""
         launch_docker_bash()
 
-    def pyspark(self):
-        """ """
-        launch_pyspark()
-
-    def test(self, script):
-        """
+    def local_test(self, script, *args, **kwargs):
+        """Run a script locally with s3 patch.
 
         Args:
           script:
+          *args:
+          **kwargs:
 
         Returns:
 
         """
-        local_spark_submit(script)
+        test_script_with_s3_mock_venv(script, *args, **kwargs)
+
+    def test(self, script, *args, **kwargs):
+        """Run a script inside the docker with s3 patch..
+
+        Args:
+          script:
+          *args:
+          **kwargs:
+          env: (Default value = 'docker')
+
+        Returns:
 
-    def export(self, *args):
         """
+        env = kwargs.pop("env", "docker")
+
+        assert env in ["sys", "local", "os", "poetry", "venv", "docker", "linux"]
+
+        if env in ["sys", "local", "os"]:
+            test_script_with_s3_mock_sys(script, *args, **kwargs)
+
+        if env in ["poetry", "venv"]:
+            test_script_with_s3_mock_venv(script, *args, **kwargs)
+
+        if env in ["docker", "linux"]:
+            test_script_with_s3_mock_docker(script, *args, **kwargs)
+
+    def logs(
+        self,
+        step_id="latest",
+        out_dir="logs",
+        cluster_name="",
+        name="*{env_name}*",
+        state="*",
+    ):
+        """Downloads logs for a step, and creates a summary or errors/warnings.
+
+        Args:
+          out_dir: (Default value = "logs")
+          step_id: (Default value = "")
+          cluster_name: (Default value = "")
+          name: (Default value = "*{env_name}*")
+          state: (Default value = "*")
+
+        Returns:
+
+        """
+        download_all_emr_logs(cluster_name, step_id, name, state, out_dir)
+        summarize_logs(cluster_name, step_id, name, state, out_dir)
+
+    def local(self, script, *args, **kwargs):
+        """Test locally. Outside the docker.
+
+        Args:
+          script:
+          *args:
+          **kwargs:
+
+        Returns:
+
+        """
+        self.local_test(script, *args, **kwargs)
+
+    def lint(self, *args, **kwargs):
+        """Lints the code in the current directory.
 
         Args:
           *args:
+          **kwargs:
 
         Returns:
 
         """
-        raise ValueError("This features is not available yet.")
-    
-    def download_logs(
-            self,
-            out_dir="logs",
-            step_id="",
-            cluster_name="",
-            name="*{env_name}*",
-            state="*",
-        ):
-        return download_emr_logs(cluster_name, step_id, name, state, out_dir)
+
+        lint_wd(*args, **kwargs)
+
+    def format(self):
+        """Auto-formats the code in the current directory."""
+        format_code()
+
+    def mock(self, s3_path: str, all: bool = False):
+        """Finds and downloads a part of the file.
+
+        Args:
+          self:
+          s3_path: str:
+          all: bool:  (Default value = False)
+          s3_path: str:
+          all: bool:  (Default value = False)
+
+        Returns:
+
+        """
+        if all:
+            mock_s3_folder(s3_path)
+        else:
+            mock_part_s3_folder(s3_path)
+
+    def spellcheck(self, path):
+        """
+
+        Args:
+          path:
+
+        Returns:
+
+        """
+        spell_check(path)
+
+    def add(self, *args, **kwargs):
+        """The add command adds required packages to your pyproject.toml and installs them.
+
+        Args:
+          *args:
+          **kwargs:
+
+        Returns:
+
+        """
+        os_cmd("poetry", "add", *args, **kwargs)
 
 
 def main():
